package com.zauzoo.gpsservice.controller;

import com.zauzoo.gpsservice.entity.Address;
import com.zauzoo.gpsservice.entity.Car;
import com.zauzoo.gpsservice.entity.History;
import com.zauzoo.gpsservice.entity.User;
import com.zauzoo.gpsservice.geolocation.GeolocationComponent;
import com.zauzoo.gpsservice.maps.AddressTriangulation;
import com.zauzoo.gpsservice.repository.AddressRepository;
import com.zauzoo.gpsservice.repository.CarRepository;
import com.zauzoo.gpsservice.repository.HistoryRepository;
import com.zauzoo.gpsservice.repository.UserRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.repository.query.Param;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import java.io.IOException;

/**
 * Kontrolore pro mapování požadavků na reverzní geokódování
 * @author Ondřej Chrbolka
 * @version 1.0
 *
 * */
@Controller
@RequestMapping("/address")
public class AddressController {
    private final GeolocationComponent geolocationComponent;

    private final AddressRepository addressRepository;

    private final UserRepository userRepository;

    private final HistoryRepository historyRepository;

    private final CarRepository carRepository;

    @Autowired
    public AddressController(GeolocationComponent geolocationComponent, AddressRepository addressRepository, UserRepository userRepository, HistoryRepository historyRepository, CarRepository carRepository) {
        this.geolocationComponent = geolocationComponent;
        this.addressRepository = addressRepository;
        this.userRepository = userRepository;
        this.historyRepository = historyRepository;
        this.carRepository = carRepository;
    }

    /**
     * Požadavek na revezní geokování se zadaným automobilem.
     *
     * @param latitude zeměpisná délka.
     * @param longitude zeměpisná šířka.
     * @param api api klíč.
     * @param id id automobilu.
     *
     * @return adresa automobilu.
     *
     * */
    @RequestMapping(value = "locatecar", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    public Address locateCar(@RequestParam("carid") String id, @RequestParam("latitude") Double latitude, @RequestParam("longitude") Double longitude, @RequestParam("api") String api) throws IOException {
        User user = userRepository.findByApikey(api);
        Car car = carRepository.findCarByExternalid(id);

        Address address = geolocationComponent.getAddress(latitude, longitude);
        addressRepository.save(address);
        historyRepository.save(new History(address, car, user, latitude, longitude));

        return address;
    }

    /**
     * Požadavek na reverzní geokódování
     * @param latitude zeměpisná délka.
     * @param longitude zeměpisná šířka.
     * @param api api klíč.
     *
     * @return adresa reverzního geokódování.
     *
     * */
    @RequestMapping(value = "geolocation", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    public Address getAddress(@RequestParam("latitude") Double latitude, @RequestParam("longitude") Double longitude,
                              @RequestParam("api") String api, @RequestParam(value = "radius", defaultValue = AddressTriangulation.DEFAULT_RADIUS_S) Double radius) throws IOException {
        Address address = geolocationComponent.getAddress(latitude, longitude, radius);

        User user = userRepository.findByApikey(api);
        addressRepository.save(address);
        historyRepository.save(new History(address, user, latitude, longitude));

        return address;
    }
}
