package com.zauzoo.gpsservice.controller;

import com.zauzoo.gpsservice.entity.Address;
import com.zauzoo.gpsservice.entity.Car;
import com.zauzoo.gpsservice.entity.History;
import com.zauzoo.gpsservice.repository.AddressRepository;
import com.zauzoo.gpsservice.repository.CarRepository;
import com.zauzoo.gpsservice.repository.HistoryRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.Date;
import java.util.List;

/**
 * Kontroler pro mapování požadavků na historii automobilu.
 * @author Ondřej Chrbolka
 * @version 1.0
 *
 * */
@Controller
@RequestMapping("location/history")
public class HistoryController {

    private final CarRepository carRepository;

    private final HistoryRepository historyRepository;

    @Autowired
    public HistoryController(CarRepository carRepository, HistoryRepository historyRepository) {
        this.carRepository = carRepository;
        this.historyRepository = historyRepository;
    }

    /**
     * Najde historii automobilů podle zadaného id.
     * @param from časové rozmezí od.
     * @param to časové rozmetí do.
     *
     * @return Historie automobilu.
     * */
    @RequestMapping(value = "date", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    public List<History> findAllByDate(@RequestParam("from") Date from, @RequestParam("to") Date to){
        return historyRepository.findAllByCreationDateBeforeAndCreationDateAfter(from, to);
    }

    /**
     * Najde historii automobilů podle zadaného id.
     * @param carid ID automobilu.
     *
     * @return Historie automobilu.
     * */
    @RequestMapping(value = "carid", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    public List<History> findAllByCarId(@RequestParam("carid") String carid){
        Car car = carRepository.findCarByExternalid(carid);

        return historyRepository.findAllByCar(car);
    }
}
