package com.zauzoo.gpsservice.core;

import com.zauzoo.gpsservice.entity.Tie;
import com.zauzoo.gpsservice.maps.SectorMapComponent;
import com.zauzoo.gpsservice.repository.TieRepository;
import org.jboss.logging.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Component;

import java.io.File;
import java.nio.file.Files;
import java.nio.file.Paths;


/**
 * Třída pro kontrolu mapových podkladů po startu aplikace
 * @author Ondřej Chrbolka
 * @version 1.0
 *
 **/
@Component
public class ConfigurationChecker implements ApplicationRunner {

    private final TieRepository tieRepository;

    private final SectorMapComponent mapComponent;

    private final Environment environment;

    private final Logger logger;

    /**
     * Cesta k dočasnému OSM souboru.
     * */
    public static String PATH = String.format("%s%stests%s", System.getProperty("user.home"), File.separator, File.separator);

    @Autowired
    public ConfigurationChecker(TieRepository tieRepository, SectorMapComponent mapComponent, Environment environment) {
        this.tieRepository = tieRepository;
        this.mapComponent = mapComponent;
        this.environment = environment;
        this.logger = Logger.getLogger(ConfigurationChecker.class);
    }

    @Override
    public void run(ApplicationArguments args){
        String path = environment.getProperty("openstreetmap.source.download.folder");
        if(path != null && Files.isDirectory(Paths.get(path)))
            PATH = path;
        else
            logger.warn("Nebyla zadána cesta ke stahování osm souborů. Využije se výchozí.");

        Iterable<Tie> ties = tieRepository.findAll();
        loadFromLocal(ties);
    }

    /**
     * Načte mapové podklady z lokálního úložiště.
     * @param ties mapové podklady.
     * */
    protected void loadFromLocal(final Iterable<Tie> ties){
        for (Tie tie : ties){
            if(Files.exists(Paths.get(tie.getFile()))){
                try{
                    mapComponent.processOSM(Paths.get(tie.getFile()), tie);
                }catch (Exception ex){
                    tieRepository.delete(tie);
                }
            }
        }

    }
}