package com.zauzoo.gpsservice.entity;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.util.Date;

/**
 * Entita firemního automobilu
 * @author Ondřej Chrbolka
 * @version 1.0
 *
 * */
@Entity
public class Car {
    /**
     * Identifikátor ve firemní databázi
     * */
    @Id
    @GeneratedValue
    private Long id;
    /**
     * Externí indentifikátor zakázníka
     * */
    @NotNull
    @NotEmpty
    @Size(max = 128)
    private String externalid;
    /**
     * Jméno automobilu dodané zákazníkem
     * */
    @Size(max = 128)
    private String name;
    /**
     * SPZ automobilu
     * */
    @Size(max = 32)
    private String spz;
    /**
     * Datum vytvoření zaznamu
     * */
    @NotNull
    private Date dateofcreation;
    /**
     * Vytvoření uživatelem
     * */
    @ManyToOne
    private User user;

    /**
     * Inicializace objektu
     * */
    public Car(Long id, String externalid, String name, String spz, Date dateofcreation, User user) {
        this.id = id;
        this.externalid = externalid;
        this.name = name;
        this.spz = spz;
        this.dateofcreation = dateofcreation;
        this.user = user;
    }

    /**
     * Inicializace objektu
     * */
    public Car() {
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getExternalid() {
        return externalid;
    }

    public void setExternalid(String externalID) {
        this.externalid = externalID;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getSpz() {
        return spz;
    }

    public void setSpz(String spz) {
        this.spz = spz;
    }

    public Date getDateofcreation() {
        return dateofcreation;
    }

    public void setDateofcreation(Date dateOfCreation) {
        this.dateofcreation = dateOfCreation;
    }

    public User getUser() {
        return user;
    }

    public void setUser(User user) {
        this.user = user;
    }
}
