package com.zauzoo.gpsservice.entity;

import org.openstreetmap.osmosis.core.domain.v0_6.Node;
import org.openstreetmap.osmosis.core.domain.v0_6.Tag;
import org.openstreetmap.osmosis.core.domain.v0_6.Way;

import java.util.*;

public final class TieBuilder {
    private Long Id;
    private Tie.STATUS status;
    private Map<Long, Way> ways = new HashMap<>();
    private Map<Long, Node> nodes = new HashMap<>();
    private Map<String, List<Node>> houses = new HashMap<>();
    private List<Tag> relationsNames = new ArrayList<>();
    private Map<String, List<Way>> streets = new HashMap<>();
    private Date dateOfCreation;
    private String file;
    private Double north;
    private Double east;
    private Double west;
    private Double south;
    private Integer sectorx;
    private Integer sectory;

    private TieBuilder() {
    }

    public static TieBuilder aTie() {
        return new TieBuilder();
    }

    public TieBuilder withId(Long Id) {
        this.Id = Id;
        return this;
    }

    public TieBuilder withStatus(Tie.STATUS status) {
        this.status = status;
        return this;
    }

    public TieBuilder withWays(Map<Long, Way> ways) {
        this.ways = ways;
        return this;
    }

    public TieBuilder withNodes(Map<Long, Node> nodes) {
        this.nodes = nodes;
        return this;
    }

    public TieBuilder withHouses(Map<String, List<Node>> houses) {
        this.houses = houses;
        return this;
    }

    public TieBuilder withRelationsNames(List<Tag> relationsNames) {
        this.relationsNames = relationsNames;
        return this;
    }

    public TieBuilder withStreets(Map<String, List<Way>> streets) {
        this.streets = streets;
        return this;
    }

    public TieBuilder withDateOfCreation(Date dateOfCreation) {
        this.dateOfCreation = dateOfCreation;
        return this;
    }

    public TieBuilder withFile(String file) {
        this.file = file;
        return this;
    }

    public TieBuilder withNorth(Double north) {
        this.north = north;
        return this;
    }

    public TieBuilder withEast(Double east) {
        this.east = east;
        return this;
    }

    public TieBuilder withWest(Double west) {
        this.west = west;
        return this;
    }

    public TieBuilder withSouth(Double south) {
        this.south = south;
        return this;
    }

    public TieBuilder withSectorx(Integer sectorx) {
        this.sectorx = sectorx;
        return this;
    }

    public TieBuilder withSectory(Integer sectory) {
        this.sectory = sectory;
        return this;
    }

    public Tie build() {
        Tie tie = new Tie();
        tie.setId(Id);
        tie.setStatus(status);
        tie.setWays(ways);
        tie.setNodes(nodes);
        tie.setHouses(houses);
        tie.setRelationsNames(relationsNames);
        tie.setStreets(streets);
        tie.setDateOfCreation(dateOfCreation);
        tie.setFile(file);
        tie.setNorth(north);
        tie.setEast(east);
        tie.setWest(west);
        tie.setSouth(south);
        tie.setSectorx(sectorx);
        tie.setSectory(sectory);
        return tie;
    }
}
