package com.zauzoo.gpsservice.entity;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import net.minidev.json.annotate.JsonIgnore;
import org.hibernate.annotations.ColumnDefault;
import org.hibernate.annotations.CreationTimestamp;
import org.springframework.boot.context.properties.bind.DefaultValue;

import javax.persistence.*;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * Uživatel API
 * @author Ondřej Chrbolka
 * @version 1.0
 * */
@JsonIgnoreProperties(value = {"cars"})
@Entity
public class User {
    /**
     * Jedinečný identifikátor
     * */
    @Id
    @GeneratedValue
    private Long id;
    /**
     * Tajný API klíč
     * */
    @NotNull
    @NotEmpty(message = "Api klíč nemůže být prázdný.")
    /*@Size(min = 64, max = 64)*/
    private String apikey;
    /**
     * Počet přístupů při používání API
     * */
    @NotNull
    @ColumnDefault("0")
    private Long numberofaccess;
    /**
     * Maximální počet přístupů
     * */
    @NotNull
    @ColumnDefault("100")
    private Long maxofaccess;
    /**
     * Datum vytvoření
     * */
    @CreationTimestamp
    private Date dateofcreation;
    /**
     * Indikace blokovaného uživatele
     * */
    @NotNull
    private boolean blocked;
    /**
     * Poznámka
     * */
    private String note;
    /**
     * Automobily uživatele
     * */
    @OneToMany(mappedBy = "user", cascade = CascadeType.ALL)
    private List<Car> cars = new ArrayList<>();

    /**
     * Inicializace objektu
     * */
    public User(Long id, String apikey, Long numberofaccess, Long maxofaccess, Date dateofcreation, boolean blocked, String note) {
        this.id = id;
        this.apikey = apikey;
        this.numberofaccess = numberofaccess;
        this.maxofaccess = maxofaccess;
        this.dateofcreation = dateofcreation;
        this.blocked = blocked;
        this.note = note;
    }

    /**
     * Inicializace objektu
     * */
    public User() {
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getApikey() {
        return apikey;
    }

    public void setApikey(String apikey) {
        this.apikey = apikey;
    }

    public Long getNumberofaccess() {
        return numberofaccess;
    }

    public void setNumberofaccess(Long numberOfAccess) {
        this.numberofaccess = numberOfAccess;
    }

    public Long getMaxofaccess() {
        return maxofaccess;
    }

    public void setMaxofaccess(Long maxOfAccess) {
        this.maxofaccess = maxOfAccess;
    }

    public Date getDateofcreation() {
        return dateofcreation;
    }

    public void setDateofcreation(Date dateOfCreation) {
        this.dateofcreation = dateOfCreation;
    }

    public boolean isBlocked() {
        return blocked;
    }

    public void setBlocked(boolean blocked) {
        this.blocked = blocked;
    }

    public String getNote() {
        return note;
    }

    public void setNote(String note) {
        this.note = note;
    }

    public List<Car> getCars() {
        return cars;
    }

    public void setCars(List<Car> cars) {
        this.cars = cars;
    }
}
