package com.zauzoo.gpsservice.geolocation;

import com.zauzoo.gpsservice.core.ConfigurationChecker;
import com.zauzoo.gpsservice.entity.Address;
import com.zauzoo.gpsservice.entity.Tie;
import com.zauzoo.gpsservice.maps.AddressTriangulation;
import com.zauzoo.gpsservice.maps.SectorMapComponent;
import com.zauzoo.gpsservice.repository.TieRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.validation.constraints.NotNull;
import java.io.IOException;
import java.util.Calendar;

/**
 * Komponenta pro správu dotazů nad mapovými podklady.
 * @author Ondřej Chrbolka
 * @version 1.0
 *
 * */
@Component
public class GeolocationComponent {
    /**
     * Komponenta pro stahování mapových podkladů.
     * */
    private final SectorMapComponent mapComponent;

    /**
     * Komponenta pro mapovou triangulaci.
     * */
    private final AddressTriangulation triangulation;

    /**
     * Úložiště mapových podkladů.
     * */
    private final TieRepository tieRepository;

    /**
     * Incializace.
     * @param mapComponent Komponenta pro stahování mapových podkladů.
     * @param tieRepository */
    @Autowired
    public GeolocationComponent(SectorMapComponent mapComponent, AddressTriangulation triangulation, TieRepository tieRepository) {
        this.mapComponent = mapComponent;
        this.triangulation = triangulation;
        this.tieRepository = tieRepository;
    }

    /**
     * Vrátí adresu na základě zadaného bodu.
     *
     * @param latitude zeměpisná šířka.
     * @param longitude zeměpisná délka.
     *
     * @return adresa lokace.
     * */
    public Address getAddress(@NotNull final Double latitude,@NotNull final Double longitude) throws IOException {
        Tie way = mapComponent.download(longitude, latitude, ConfigurationChecker.PATH);

        way.setStatus(Tie.STATUS.IMPORTING);
        way.setDateOfCreation(Calendar.getInstance().getTime());

        this.tieRepository.save(way);
        way.setStatus(Tie.STATUS.COMPLETE);

        Address possible = triangulation.findPossibleAddress(way, latitude, longitude);

        return possible;
    }

    public Address getAddress(Double latitude, Double longitude, Double radius) throws IOException {
        Tie way = mapComponent.download(longitude, latitude, ConfigurationChecker.PATH);

        way.setStatus(Tie.STATUS.IMPORTING);
        way.setDateOfCreation(Calendar.getInstance().getTime());

        this.tieRepository.save(way);
        way.setStatus(Tie.STATUS.COMPLETE);

        Address possible = triangulation.findPossibleAddress(way, latitude, longitude, radius);

        return possible;

    }
}
