package com.zauzoo.gpsservice.maps;

import com.zauzoo.gpsservice.entity.Tie;

import javax.validation.constraints.NotNull;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.file.Path;

/**
 * Rozhraní pro stahování a správu OpenStreetMap
 *
 * @author Ondřej Chrbolka
 * @version 1.0
 *
 * */
public interface MapComponent {
    /**
     * Stáhne osm soubor s mapovou oblastí do určené lokace.
     *
     * @param longitude zeměpisná délka
     * @param latitude zeměpisná šířka
     * @param location Umístění na serveru, kde bude soubor uchován k dalšímu zpracování.
     * @throws IOException Vstupně výstupní chyba stahování.
     *
     * @return*/
    Tie download(Double longitude, Double latitude, String location) throws IOException;

    /**
     * Zpracuje OSM soubor do cest.
     * @param location Umístění na serveru, kde se nachází OSM soubor.
     * @param tie Mapový podklad
     * @throws FileNotFoundException Vstupně výstupní chyba souboru.
     *
     * @return mapa cest
     *
     * */
    Tie processOSM(final Path location, Tie tie) throws FileNotFoundException;
}
