package com.zauzoo.gpsservice.maps;

import com.zauzoo.gpsservice.entity.Address;
import com.zauzoo.gpsservice.entity.Tie;

import junit.framework.TestCase;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.mockito.Spy;
import org.mockito.junit.MockitoJUnitRunner;
import org.springframework.core.env.Environment;

import java.io.FileNotFoundException;
import java.net.URISyntaxException;
import java.nio.file.Paths;

import static org.junit.Assert.*;

@RunWith(MockitoJUnitRunner.class)
public class AddressTriangulationTest extends TestCase {

    @Spy
    @InjectMocks
    private OsmMapComponent mapComponent;

    @Mock
    private Environment environment;

    @Spy
    @InjectMocks
    private AddressTriangulation addressTriangulation;

    @Before
    public void setUp() throws Exception {
        MockitoAnnotations.initMocks(this);
    }

    @After
    public void tearDown() throws Exception {
    }

    @Test
    public void getWay() throws FileNotFoundException, URISyntaxException {
        final Double posLat = 49.60733;
        final Double posLon = 15.5794;

        Tie tie = new Tie();

        mapComponent.processOSM(Paths.get("src/test/resources/map-test1.osm"), tie);
        Object w = addressTriangulation.findPossibleAddress(tie, posLat, posLon);

        assertNotEquals(tie.getWays(), 0);
        assertNotEquals(tie.getNodes(), 0);
        assertNotEquals(tie.getHouses(), 0);
        assertNotNull(w);
    }

    @Test
    public void getAddressVillage() throws FileNotFoundException {
        final Double posLat = 49.54323;
        final Double posLon = 15.59517;

        Tie tie = new Tie();

        mapComponent.processOSM(Paths.get("src/test/resources/village.osm"), tie);
        Address a = addressTriangulation.findPossibleAddress(tie, posLat, posLon);

        assertEquals("", a.getStreet());
        assertEquals("Havlíčkův Brod", a.getCity());
        assertEquals("Květnov", a.getPlace());
        assertEquals("35", a.getNumber());
        assertEquals("58001", a.getZip());
    }

    @Test
    public void getAddressSmallCity() throws FileNotFoundException {
        final Double posLat = 49.54104;
        final Double posLon = 15.35897;

        Tie tie = new Tie();

        mapComponent.processOSM(Paths.get("src/test/resources/smallcity.osm"), tie);
        Address a = addressTriangulation.findPossibleAddress(tie, posLat, posLon);

        assertEquals("Horní náměstí", a.getStreet());
        assertEquals("Humpolec", a.getCity());
        assertEquals("Humpolec", a.getPlace());
        assertEquals("8", a.getNumber());
        assertEquals("39601", a.getZip());
    }

    @Test
    public void getAddressCity() throws FileNotFoundException {
        final Double posLat = 50.23057;
        final Double posLon = 12.86861;

        Tie tie = new Tie();

        mapComponent.processOSM(Paths.get("src/test/resources/city.osm"), tie);
        Address a = addressTriangulation.findPossibleAddress(tie, posLat, posLon);

        assertEquals("Varšavská", a.getStreet());
        assertEquals("Karlovy Vary", a.getCity());
        assertEquals("Karlovy Vary", a.getPlace());
        assertEquals("929/15", a.getNumber());
        assertEquals("36001", a.getZip());
    }

    @Test
    public void getAddressBigCity() throws FileNotFoundException {
        final Double posLat = 50.08177;
        final Double posLon = 14.42626;

        Tie tie = new Tie();

        mapComponent.processOSM(Paths.get("src/test/resources/bigcity.osm"), tie);
        Address a = addressTriangulation.findPossibleAddress(tie, posLat, posLon);

        assertEquals("Václavské náměstí", a.getStreet());
        assertEquals("Praha", a.getCity());
        assertEquals("Nové Město", a.getPlace());
        assertEquals("36", a.getNumber());
        assertEquals("11000", a.getZip());
    }
}